@@ -33,6 +33,7 @@
 #include <QTemporaryFile>
 #include <QTest>
 #include <QUrl>
+#include <QElapsedTimer>
 
 #include <kmountpoint.h>
 #include <kprotocolinfo.h>
@@ -69,22 +70,6 @@ static bool otherTmpDirIsOnSamePartition() // true on CI because it's a LXC cont
     return srcMountPoint->mountedFrom() == destMountPoint->mountedFrom();
 }
 
-#if 0
-static QUrl systemTmpDir()
-{
-#ifdef Q_OS_WIN
-    return QUrl("system:" + QDir::homePath() + "/.kde-unit-test/jobtest-system/");
-#else
-    return QUrl("system:/home/.kde-unit-test/jobtest-system/");
-#endif
-}
-
-static QString realSystemPath()
-{
-    return QStandardPaths::writableLocation(QStandardPaths::DataLocation) + "/jobtest-system/";
-}
-#endif
-
 void JobTest::initTestCase()
 {
     QStandardPaths::setTestModeEnabled(true);
@@ -104,16 +89,6 @@ void JobTest::initTestCase()
             qFatal("couldn't create %s", qPrintable(otherTmpDir()));
         }
     }
-#if 0
-    if (KProtocolInfo::isKnownProtocol("system")) {
-        if (!QFile::exists(realSystemPath())) {
-            bool ok = dir.mkdir(realSystemPath());
-            if (!ok) {
-                qFatal("couldn't create %s", qPrintable(realSystemPath()));
-            }
-        }
-    }
-#endif
 
     qRegisterMetaType<KJob *>("KJob*");
     qRegisterMetaType<KIO::Job *>("KIO::Job*");
@@ -124,11 +99,6 @@ void JobTest::cleanupTestCase()
 {
     QDir(homeTmpDir()).removeRecursively();
     QDir(otherTmpDir()).removeRecursively();
-#if 0
-    if (KProtocolInfo::isKnownProtocol("system")) {
-        delDir(systemTmpDir());
-    }
-#endif
 }
 
 void JobTest::enterLoop()
@@ -226,7 +196,7 @@ void JobTest::storedPut()
     QDateTime mtime = QDateTime::fromSecsSinceEpoch(secsSinceEpoch - 30); // 30 seconds ago
     job->setModificationTime(mtime);
     job->setUiDelegate(nullptr);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QFileInfo fileInfo(filePath);
     QVERIFY(fileInfo.exists());
     QCOMPARE(fileInfo.size(), (long long)putData.size());
@@ -248,7 +218,7 @@ void JobTest::storedPutIODevice()
     QDateTime mtime = QDateTime::fromSecsSinceEpoch(secsSinceEpoch - 30); // 30 seconds ago
     job->setModificationTime(mtime);
     job->setUiDelegate(nullptr);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QFileInfo fileInfo(filePath);
     QVERIFY(fileInfo.exists());
     QCOMPARE(fileInfo.size(), (long long)putData.size());
@@ -330,7 +300,7 @@ void JobTest::storedPutIODeviceFastDevice()
     putDataBuffer.seek(pos);
     putDataBuffer.readChannelFinished();
 
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QCOMPARE(size, putDataContents.size());
     QCOMPARE(putDataBuffer.bytesAvailable(), 0);
 
@@ -373,7 +343,7 @@ void JobTest::storedPutIODeviceSlowDevice()
     // Simulate the transfer is done
     QTimer::singleShot(450, this, [&putDataBuffer](){ putDataBuffer.readChannelFinished(); });
 
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QCOMPARE(size, putDataContents.size() * 2);
     QCOMPARE(putDataBuffer.bytesAvailable(), 0);
 
@@ -415,7 +385,7 @@ void JobTest::storedPutIODeviceSlowDeviceBigChunk()
     // Simulate the transfer is done
     QTimer::singleShot(450, this, [&putDataBuffer](){ putDataBuffer.readChannelFinished(); });
 
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QCOMPARE(size, putDataContents.size());
     QCOMPARE(putDataBuffer.bytesAvailable(), 0);
 
@@ -463,8 +433,7 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     // copy the file with file_copy
     KIO::Job *job = KIO::file_copy(u, d, perms, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // still there
     QCOMPARE(int(QFileInfo(dest).permissions()), int(0x6666));
@@ -490,8 +459,7 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     QSignalSpy spyCopyingDone(job, SIGNAL(copyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)));
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // still there
     {
@@ -513,7 +481,7 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     job = KIO::copyAs(u, d, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // still there
 
@@ -521,7 +489,7 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     job = KIO::copyAs(u, d, KIO::Overwrite | KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // still there
 
@@ -551,8 +519,7 @@ void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int f
     KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFileInfo(dest).isDir());
     QVERIFY(QFileInfo(dest + "/testfile").isFile());
@@ -578,15 +545,13 @@ void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int f
     job = KIO::copyAs(u, d, KIO::HideProgressInfo | KIO::Overwrite);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     // Do it again, without Overwrite (should fail).
     job = KIO::copyAs(u, d, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    ok = job->exec();
-    QVERIFY(!ok);
+    QVERIFY(!job->exec());
 }
 
 #ifndef Q_OS_WIN
@@ -717,7 +682,7 @@ void JobTest::copyFolderWithUnaccessibleSubfolder()
 
     QSignalSpy spy(job, SIGNAL(warning(KJob*,QString,QString)));
     job->setUiDelegate(nullptr);   // no skip dialog, thanks
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     QFile(inaccessible).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
 
@@ -739,7 +704,7 @@ void JobTest::copyDataUrl()
     QVERIFY(!QFileInfo::exists(dst_dir + "/data"));
     // WHEN
     KIO::CopyJob *job = KIO::copy(QUrl("data:,Hello%2C%20World!"), QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     // THEN
     QVERIFY(QFileInfo(dst_dir + "/data").isFile());
     QFile::remove(dst_dir + "/data");
@@ -760,7 +725,7 @@ void JobTest::suspendFileCopy()
     QVERIFY(job->suspend());
     QVERIFY(!spyResult.wait(300));
     QVERIFY(job->resume());
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QFile::remove(dest);
 }
@@ -780,7 +745,7 @@ void JobTest::suspendCopy()
     QVERIFY(job->suspend());
     QVERIFY(!spyResult.wait(300));
     QVERIFY(job->resume());
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QFile::remove(dest);
 }
@@ -794,8 +759,7 @@ void JobTest::moveLocalFile(const QString &src, const QString &dest)
     // move the file with file_move
     KIO::Job *job = KIO::file_move(u, d, 0666, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(!QFile::exists(src));     // not there anymore
     QCOMPARE(int(QFileInfo(dest).permissions()), int(0x6666));
@@ -804,8 +768,7 @@ void JobTest::moveLocalFile(const QString &src, const QString &dest)
     job = KIO::move(d, u, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // it's back
 }
@@ -821,11 +784,7 @@ static void moveLocalSymlink(const QString &src, const QString &dest)
     KIO::Job *job = KIO::move(u, d, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    bool ok = job->exec();
-    if (!ok) {
-        qWarning() << job->error();
-    }
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QT_LSTAT(QFile::encodeName(dest).constData(), &buf) == 0);
     QVERIFY(!QFile::exists(src));     // not there anymore
 
@@ -833,8 +792,7 @@ static void moveLocalSymlink(const QString &src, const QString &dest)
     job = KIO::move(d, u, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QT_LSTAT(QFile::encodeName(dest).constData(), &buf) != 0); // doesn't exist anymore
     QVERIFY(QT_LSTAT(QFile::encodeName(src).constData(), &buf) == 0); // it's back
 }
@@ -854,8 +812,7 @@ void JobTest::moveLocalDirectory(const QString &src, const QString &dest)
     KIO::Job *job = KIO::move(u, d, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    bool ok = job->exec();
-    QVERIFY2(ok, qPrintable(job->errorString()));
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFileInfo(dest).isDir());
     QVERIFY(QFileInfo(dest + "/testfile").isFile());
@@ -994,6 +951,72 @@ void JobTest::moveDirectoryNoPermissions()
     QVERIFY(QDir(subdir).removeRecursively());
 }
 
+void JobTest::moveDirectoryToReadonlyFilesystem_data()
+{
+    QTest::addColumn<QList<QUrl>>("sources");
+    QTest::addColumn<int>("expectedErrorCode");
+
+    const QString srcFileHomePath = homeTmpDir() + "srcFileHome";
+    const QUrl srcFileHome = QUrl::fromLocalFile(srcFileHomePath);
+    createTestFile(srcFileHomePath);
+
+    const QString srcFileOtherPath = otherTmpDir() + "srcFileOther";
+    const QUrl srcFileOther = QUrl::fromLocalFile(srcFileOtherPath);
+    createTestFile(srcFileOtherPath);
+
+    const QString srcDirHomePath = homeTmpDir() + "srcDirHome";
+    const QUrl srcDirHome = QUrl::fromLocalFile(srcDirHomePath);
+    createTestDirectory(srcDirHomePath);
+
+    const QString srcDirHome2Path = homeTmpDir() + "srcDirHome2";
+    const QUrl srcDirHome2 = QUrl::fromLocalFile(srcDirHome2Path);
+    createTestDirectory(srcDirHome2Path);
+
+    const QString srcDirOtherPath = otherTmpDir() + "srcDirOther";
+    const QUrl srcDirOther = QUrl::fromLocalFile(srcDirOtherPath);
+    createTestDirectory(srcDirOtherPath);
+
+    QTest::newRow("file_same_partition") << QList<QUrl>{srcFileHome} << int(KIO::ERR_WRITE_ACCESS_DENIED);
+    QTest::newRow("file_other_partition") << QList<QUrl>{srcFileOther} << int(KIO::ERR_WRITE_ACCESS_DENIED);
+    QTest::newRow("one_dir_same_partition") << QList<QUrl>{srcDirHome} << int(KIO::ERR_WRITE_ACCESS_DENIED);
+    QTest::newRow("one_dir_other_partition") << QList<QUrl>{srcDirOther} << int(KIO::ERR_WRITE_ACCESS_DENIED);
+    QTest::newRow("dirs_same_partition") << QList<QUrl>{srcDirHome, srcDirHome2} << int(KIO::ERR_WRITE_ACCESS_DENIED);
+    QTest::newRow("dirs_both_partitions") << QList<QUrl>{srcDirOther, srcDirHome} << int(KIO::ERR_WRITE_ACCESS_DENIED);
+}
+
+void JobTest::moveDirectoryToReadonlyFilesystem()
+{
+    QFETCH(QList<QUrl>, sources);
+    QFETCH(int, expectedErrorCode);
+
+    const QString dst_dir = homeTmpDir() + "readonlyDest";
+    const QUrl dst = QUrl::fromLocalFile(dst_dir);
+    QVERIFY2(QDir().mkdir(dst_dir), qPrintable(dst_dir));
+    QFile(dst_dir).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::ExeOwner)); // Make it readonly, moving should throw some errors
+
+    KIO::CopyJob *job = KIO::move(sources, dst, KIO::HideProgressInfo | KIO::NoPrivilegeExecution);
+    job->setUiDelegate(nullptr);
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), expectedErrorCode);
+    for (const QUrl &srcUrl : qAsConst(sources)) {
+        QVERIFY(QFileInfo::exists(srcUrl.toLocalFile())); // no moving happened
+    }
+
+    KIO::CopyJob *job2 = KIO::move(sources, dst, KIO::HideProgressInfo);
+    job2->setUiDelegate(nullptr);
+    QVERIFY(!job2->exec());
+    if (job2->error() != KIO::ERR_CANNOT_MKDIR) { // This can happen when moving between partitions, but on CI it's the same partition so allow both
+        QCOMPARE(job2->error(), expectedErrorCode);
+    }
+    for (const QUrl &srcUrl : qAsConst(sources)) {
+        QVERIFY(QFileInfo::exists(srcUrl.toLocalFile())); // no moving happened
+    }
+
+    // Cleanup
+    QVERIFY(QFile(dst_dir).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner)));
+    QVERIFY(QDir(dst_dir).removeRecursively());
+}
+
 void JobTest::listRecursive()
 {
     // Note: many other tests must have been run before since we rely on the files they created
@@ -1008,8 +1031,7 @@ void JobTest::listRecursive()
     job->setUiDelegate(nullptr);
     connect(job, &KIO::ListJob::entries,
             this, &JobTest::slotEntries);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     m_names.sort();
     QByteArray ref_names = QByteArray(".,..,"
                                       "dirFromHome,dirFromHome/testfile,"
@@ -1024,7 +1046,7 @@ void JobTest::listRecursive()
 #endif
                                       "fileFromHome");
 
-    const QString joinedNames = m_names.join(QStringLiteral(","));
+    const QString joinedNames = m_names.join(QLatin1Char(','));
     if (joinedNames.toLatin1() != ref_names) {
         qDebug("%s", qPrintable(joinedNames));
         qDebug("%s", ref_names.data());
@@ -1093,8 +1115,7 @@ void JobTest::directorySize()
 
     KIO::DirectorySizeJob *job = KIO::directorySize(QUrl::fromLocalFile(src));
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     qDebug() << "totalSize: " << job->totalSize();
     qDebug() << "totalFiles: " << job->totalFiles();
     qDebug() << "totalSubdirs: " << job->totalSubdirs();
@@ -1115,8 +1136,7 @@ void JobTest::directorySizeError()
 {
     KIO::DirectorySizeJob *job = KIO::directorySize(QUrl::fromLocalFile(QStringLiteral("/I/Dont/Exist")));
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(!ok);
+    QVERIFY(!job->exec());
     qApp->sendPostedEvents(nullptr, QEvent::DeferredDelete);
 }
 
@@ -1142,82 +1162,6 @@ void JobTest::calculateRemainingSeconds()
     QCOMPARE(text, i18n("00:00:50"));
 }
 
-#if 0
-void JobTest::copyFileToSystem()
-{
-    if (!KProtocolInfo::isKnownProtocol("system")) {
-        qDebug() << "no kio_system, skipping test";
-        return;
-    }
-
-    // First test with support for UDS_LOCAL_PATH
-    copyFileToSystem(true);
-
-    QString dest = realSystemPath() + "fileFromHome_copied";
-    QFile::remove(dest);
-
-    // Then disable support for UDS_LOCAL_PATH, i.e. test what would
-    // happen for ftp, smb, http etc.
-    copyFileToSystem(false);
-}
-
-void JobTest::copyFileToSystem(bool resolve_local_urls)
-{
-    qDebug() << resolve_local_urls;
-    extern KIOCORE_EXPORT bool kio_resolve_local_urls;
-    kio_resolve_local_urls = resolve_local_urls;
-
-    const QString src = homeTmpDir() + "fileFromHome";
-    createTestFile(src);
-    QUrl u = QUrl::fromLocalFile(src);
-    QUrl d = QUrl::fromLocalFile(systemTmpDir());
-    d.addPath("fileFromHome_copied");
-
-    qDebug() << "copying " << u << " to " << d;
-
-    // copy the file with file_copy
-    m_mimetype.clear();
-    KIO::FileCopyJob *job = KIO::file_copy(u, d, -1, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
-            this, SLOT(slotMimetype(KIO::Job*,QString)));
-    bool ok = job->exec();
-    QVERIFY(ok);
-
-    QString dest = realSystemPath() + "fileFromHome_copied";
-
-    QVERIFY(QFile::exists(dest));
-    QVERIFY(QFile::exists(src));     // still there
-
-    {
-        // do NOT check that the timestamp is the same.
-        // It can't work with file_copy when it uses the datapump,
-        // unless we use setModificationTime in the app code.
-    }
-
-    // Check mimetype
-    QCOMPARE(m_mimetype, QString("text/plain"));
-
-    // cleanup and retry with KIO::copy()
-    QFile::remove(dest);
-    job = KIO::copy(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    ok = job->exec();
-    QVERIFY(ok);
-    QVERIFY(QFile::exists(dest));
-    QVERIFY(QFile::exists(src));     // still there
-    {
-        // check that the timestamp is the same (#79937)
-        QFileInfo srcInfo(src);
-        QFileInfo destInfo(dest);
-        QCOMPARE(srcInfo.lastModified(), destInfo.lastModified());
-    }
-
-    // restore normal behavior
-    kio_resolve_local_urls = true;
-}
-#endif
-
 void JobTest::getInvalidUrl()
 {
     QUrl url(QStringLiteral("http://strange<hostname>/"));
@@ -1229,8 +1173,7 @@ void JobTest::getInvalidUrl()
 
     KIO::Scheduler::setJobPriority(job, 1); // shouldn't crash (#135456)
 
-    bool ok = job->exec();
-    QVERIFY(!ok);   // it should fail :)
+    QVERIFY(!job->exec());   // it should fail :)
 }
 
 void JobTest::slotMimetype(KIO::Job *job, const QString &type)
@@ -1245,8 +1188,7 @@ void JobTest::deleteFile()
     createTestFile(dest);
     KIO::Job *job = KIO::del(QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(dest));
 }
 
@@ -1272,8 +1214,7 @@ void JobTest::deleteDirectory()
 
     KIO::Job *job = KIO::del(QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(dest));
 }
 
@@ -1295,8 +1236,7 @@ void JobTest::deleteSymlink(bool using_fast_path)
     }
     KIO::Job *job = KIO::del(QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(dest));
     QVERIFY(QFile::exists(src));
 #endif
@@ -1324,13 +1264,12 @@ void JobTest::deleteManyDirs(bool using_fast_path)
         createTestDirectory(dir);
         dirs << QUrl::fromLocalFile(dir);
     }
-    QTime dt;
+    QElapsedTimer dt;
     dt.start();
     KIO::Job *job = KIO::del(dirs, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
-    Q_FOREACH (const QUrl &dir, dirs) {
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    for (const QUrl &dir : qAsConst(dirs)) {
         QVERIFY(!QFile::exists(dir.toLocalFile()));
     }
 
@@ -1363,7 +1302,7 @@ static QList<QUrl> createManyFiles(const QString &baseDir, int numFiles)
 
 void JobTest::deleteManyFilesIndependently()
 {
-    QTime dt;
+    QElapsedTimer dt;
     dt.start();
     const int numFiles = 100; // Use 1000 for performance testing
     const QString baseDir = homeTmpDir();
@@ -1377,8 +1316,7 @@ void JobTest::deleteManyFilesIndependently()
         //qDebug() << file;
         KIO::Job *job = KIO::del(url, KIO::HideProgressInfo);
         job->setUiDelegate(nullptr);
-        bool ok = job->exec();
-        QVERIFY(ok);
+        QVERIFY2(job->exec(), qPrintable(job->errorString()));
         QVERIFY(!QFile::exists(file));
     }
     qDebug() << "Deleted" << numFiles << "files in" << dt.elapsed() << "milliseconds";
@@ -1389,7 +1327,7 @@ void JobTest::deleteManyFilesTogether(bool using_fast_path)
     extern KIOCORE_EXPORT bool kio_resolve_local_urls;
     kio_resolve_local_urls = !using_fast_path;
 
-    QTime dt;
+    QElapsedTimer dt;
     dt.start();
     const int numFiles = 100; // Use 1000 for performance testing
     const QString baseDir = homeTmpDir();
@@ -1399,8 +1337,7 @@ void JobTest::deleteManyFilesTogether(bool using_fast_path)
     //qDebug() << file;
     KIO::Job *job = KIO::del(urls, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     qDebug() << "Deleted" << numFiles << "files in" << dt.elapsed() << "milliseconds";
 
     kio_resolve_local_urls = true;
@@ -1418,7 +1355,7 @@ void JobTest::rmdirEmpty()
     QDir().mkdir(dir);
     QVERIFY(QFile::exists(dir));
     KIO::Job *job = KIO::rmdir(QUrl::fromLocalFile(dir));
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(dir));
 }
 
@@ -1440,8 +1377,7 @@ void JobTest::stat()
     const QUrl url(QUrl::fromLocalFile(filePath));
     KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
     QVERIFY(job);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     // TODO set setSide, setDetails
     const KIO::UDSEntry &entry = job->statResult();
     QVERIFY(!entry.isDir());
@@ -1465,8 +1401,7 @@ void JobTest::stat()
     // Testing stat over HTTP
     KIO::StatJob *job = KIO::stat(QUrl("http://www.kde.org"), KIO::HideProgressInfo);
     QVERIFY(job);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     // TODO set setSide, setDetails
     const KIO::UDSEntry &entry = job->statResult();
     QVERIFY(!entry.isDir());
@@ -1488,8 +1423,7 @@ void JobTest::statSymlink()
     const QUrl url(QUrl::fromLocalFile(symlink));
     KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
     QVERIFY(job);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     // TODO set setSide, setDetails
     const KIO::UDSEntry &entry = job->statResult();
     QVERIFY(!entry.isDir());
@@ -1520,8 +1454,7 @@ void JobTest::mostLocalUrl()
     createTestFile(filePath);
     KIO::StatJob *job = KIO::mostLocalUrl(QUrl::fromLocalFile(filePath), KIO::HideProgressInfo);
     QVERIFY(job);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QCOMPARE(job->mostLocalUrl().toLocalFile(), filePath);
 }
 
@@ -1536,7 +1469,7 @@ void JobTest::chmodFile()
     KFileItemList items; items << item;
     KIO::Job *job = KIO::chmod(items, newPerm, S_IWGRP /*TODO: QFile::WriteGroup*/, QString(), QString(), false, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     KFileItem newItem(QUrl::fromLocalFile(filePath));
     QCOMPARE(QString::number(newItem.permissions(), 8), QString::number(newPerm, 8));
@@ -1555,7 +1488,7 @@ void JobTest::chmodSticky()
     KFileItemList items({item});
     KIO::Job *job = KIO::chmod(items, newPerm, S_ISVTX, QString(), QString(), false, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     KFileItem newItem(QUrl::fromLocalFile(dirPath));
     QCOMPARE(QString::number(newItem.permissions(), 8), QString::number(newPerm, 8));
@@ -1576,10 +1509,10 @@ void JobTest::chmodFileError()
     KIO::Job *job = KIO::chmod(items, newPerm, S_IWGRP /*TODO: QFile::WriteGroup*/, QStringLiteral("root"), QString(), false, KIO::HideProgressInfo);
     // Simulate the user pressing "Skip" in the dialog.
     PredefinedAnswerJobUiDelegate extension;
-    extension.m_skipResult = KIO::S_SKIP;
+    extension.m_skipResult = KIO::Result_Skip;
     job->setUiDelegateExtension(&extension);
 
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     QCOMPARE(extension.m_askSkipCalled, 1);
     KFileItem newItem(QUrl::fromLocalFile(filePath));
@@ -1596,8 +1529,7 @@ void JobTest::mimeType()
     KIO::MimetypeJob *job = KIO::mimetype(QUrl::fromLocalFile(filePath), KIO::HideProgressInfo);
     QVERIFY(job);
     QSignalSpy spyMimeType(job, SIGNAL(mimetype(KIO::Job*,QString)));
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QCOMPARE(spyMimeType.count(), 1);
     QCOMPARE(spyMimeType[0][0], QVariant::fromValue(static_cast<KIO::Job *>(job)));
     QCOMPARE(spyMimeType[0][1].toString(), QStringLiteral("application/octet-stream"));
@@ -1606,8 +1538,7 @@ void JobTest::mimeType()
     KIO::MimetypeJob *job = KIO::mimetype(QUrl("http://www.kde.org"), KIO::HideProgressInfo);
     QVERIFY(job);
     QSignalSpy spyMimeType(job, SIGNAL(mimetype(KIO::Job*,QString)));
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QCOMPARE(spyMimeType.count(), 1);
     QCOMPARE(spyMimeType[0][0], QVariant::fromValue(static_cast<KIO::Job *>(job)));
     QCOMPARE(spyMimeType[0][1].toString(), QString("text/html"));
@@ -1622,8 +1553,7 @@ void JobTest::mimeTypeError()
     QVERIFY(job);
     QSignalSpy spyMimeType(job, SIGNAL(mimetype(KIO::Job*,QString)));
     QSignalSpy spyResult(job, SIGNAL(result(KJob*)));
-    bool ok = job->exec();
-    QVERIFY(!ok);
+    QVERIFY(!job->exec());
     QCOMPARE(spyMimeType.count(), 0);
     QCOMPARE(spyResult.count(), 1);
 }
@@ -1642,8 +1572,7 @@ void JobTest::moveFileDestAlreadyExists() // #157601
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
     job->setAutoSkip(true);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(file1)); // it was skipped
     QVERIFY(!QFile::exists(file2)); // it was moved
 }
@@ -1676,7 +1605,7 @@ void JobTest::moveDestAlreadyExistsAutoRename()
     if (samePartition) {
         // cleanup
         KIO::Job *job = KIO::del(QUrl::fromLocalFile(dir), KIO::HideProgressInfo);
-        QVERIFY(job->exec());
+        QVERIFY2(job->exec(), qPrintable(job->errorString()));
         QVERIFY(!QFile::exists(dir));
     }
 }
@@ -1684,13 +1613,13 @@ void JobTest::moveDestAlreadyExistsAutoRename()
 void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveDirs) // #256650
 {
     const QString prefix = moveDirs ? QStringLiteral("dir ") : QStringLiteral("file ");
-    QStringList sources;
+
     const QString file1 = homeTmpDir() + prefix + "(1)";
     const QString file2 = homeTmpDir() + prefix + "(2)";
     const QString existingDest1 = destDir + prefix + "(1)";
     const QString existingDest2 = destDir + prefix + "(2)";
-    sources << file1 << file2 << existingDest1 << existingDest2;
-    Q_FOREACH (const QString &source, sources) {
+    const QStringList sources = QStringList() << file1 << file2 << existingDest1 << existingDest2;
+    for (const QString &source : sources) {
         if (moveDirs) {
             QVERIFY(QDir().mkdir(source));
         } else {
@@ -1706,10 +1635,9 @@ void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveD
 
     //qDebug() << QDir(destDir).entryList();
 
-    bool ok = job->exec();
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     qDebug() << QDir(destDir).entryList();
-    QVERIFY(ok);
     QVERIFY(!QFile::exists(file1)); // it was moved
     QVERIFY(!QFile::exists(file2)); // it was moved
     QVERIFY(QFile::exists(existingDest1));
@@ -1731,6 +1659,37 @@ void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveD
     }
 }
 
+void JobTest::copyDirectoryAlreadyExistsSkip()
+{
+    // when copying a directory (which contains at least one file) to some location, and then
+    // copying the same dir to the same location again, and clicking "Skip" there should be no
+    // segmentation fault, bug 408350
+
+    const QString src = homeTmpDir() + "a";
+    createTestDirectory(src);
+    const QString dest = homeTmpDir() + "dest";
+    createTestDirectory(dest);
+
+    QUrl u = QUrl::fromLocalFile(src);
+    QUrl d = QUrl::fromLocalFile(dest);
+
+    KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
+    job->setUiDelegate(nullptr);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    QVERIFY(QFile::exists(dest + QStringLiteral("/a/testfile")));
+
+    job = KIO::copy(u, d, KIO::HideProgressInfo);
+    // Simulate the user pressing "Skip" in the dialog.
+    PredefinedAnswerJobUiDelegate extension;
+    extension.m_skipResult = KIO::Result_Skip;
+    job->setUiDelegateExtension(&extension);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    QVERIFY(QFile::exists(dest + QStringLiteral("/a/testfile")));
+
+    QDir(src).removeRecursively();
+    QDir(dest).removeRecursively();
+}
+
 void JobTest::safeOverwrite_data()
 {
     QTest::addColumn<bool>("destFileExists");
@@ -1769,10 +1728,11 @@ void JobTest::safeOverwrite()
     QSignalSpy spyTotalSize(job, &KIO::FileCopyJob::totalSize);
     connect(job, &KIO::FileCopyJob::totalSize, this, [destFileExists, destPartFile](KJob *job, qulonglong totalSize) {
         Q_UNUSED(job);
-        Q_UNUSED(totalSize);
-        QCOMPARE(destFileExists, QFile::exists(destPartFile));
+        if (totalSize > 0) {
+            QCOMPARE(destFileExists, QFile::exists(destPartFile));
+        }
     });
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFile::exists(destFile));
     QVERIFY(!QFile::exists(srcFile));
     QVERIFY(!QFile::exists(destPartFile));
@@ -1791,8 +1751,7 @@ void JobTest::moveAndOverwrite()
 
     KIO::FileCopyJob *job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
     job->setUiDelegate(nullptr);
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
 
 #ifndef Q_OS_WIN
@@ -1802,19 +1761,17 @@ void JobTest::moveAndOverwrite()
     QVERIFY(QFile::exists(existingDest));
     job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
     job->setUiDelegate(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
 
     // Now same thing when the target is a symlink to another file
     createTestFile(sourceFile);
-    createTestFile(sourceFile + "2");
-    createTestSymlink(existingDest, QFile::encodeName(sourceFile + "2"));
+    createTestFile(sourceFile + QLatin1Char('2'));
+    createTestSymlink(existingDest, QFile::encodeName(sourceFile + QLatin1Char('2')));
     QVERIFY(QFile::exists(existingDest));
     job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
     job->setUiDelegate(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
 
     // Now same thing when the target is a _broken_ symlink
@@ -1823,8 +1780,7 @@ void JobTest::moveAndOverwrite()
     QVERIFY(!QFile::exists(existingDest)); // it exists, but it's broken...
     job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
     job->setUiDelegate(nullptr);
-    ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
 #endif
 }
@@ -1841,8 +1797,7 @@ void JobTest::moveOverSymlinkToSelf() // #169547
     KIO::CopyJob *job = KIO::move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    bool ok = job->exec();
-    QVERIFY(!ok);
+    QVERIFY(!job->exec());
     QCOMPARE(job->error(), (int)KIO::ERR_FILE_ALREADY_EXIST); // and not ERR_IDENTICAL_FILES!
     QVERIFY(QFile::exists(sourceFile)); // it not moved
 #endif
@@ -1860,7 +1815,7 @@ void JobTest::createSymlink()
 
     // With KIO::link (high-level)
     KIO::CopyJob *job = KIO::link(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(destDir), KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFileInfo::exists(sourceFile));
     const QString dest = destDir + "/fileFromHome";
     QVERIFY(QFileInfo(dest).isSymLink());
@@ -1870,7 +1825,7 @@ void JobTest::createSymlink()
     // With KIO::symlink (low-level)
     const QString linkPath = destDir + "/link";
     KIO::Job *symlinkJob = KIO::symlink(sourceFile, QUrl::fromLocalFile(linkPath), KIO::HideProgressInfo);
-    QVERIFY(symlinkJob->exec());
+    QVERIFY2(symlinkJob->exec(), qPrintable(symlinkJob->errorString()));
     QVERIFY(QFileInfo::exists(sourceFile));
     QVERIFY(QFileInfo(linkPath).isSymLink());
     QCOMPARE(QFileInfo(linkPath).symLinkTarget(), sourceFile);
@@ -1904,7 +1859,7 @@ void JobTest::createSymlinkAsShouldSucceed()
     QFile::remove(dest); // just in case
 
     KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFileInfo::exists(sourceFile));
     QVERIFY(QFileInfo(dest).isSymLink());
     QVERIFY(QFile::remove(dest));
@@ -1949,7 +1904,7 @@ void JobTest::createSymlinkAsShouldFailFileExists()
 
     // First time works
     KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFileInfo(dest).isSymLink());
 
     // Second time fails (already exists)
@@ -1979,20 +1934,20 @@ void JobTest::createSymlinkWithOverwriteShouldWork()
 
     // First time works
     KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFileInfo(dest).isSymLink());
 
     // Changing the link target, with overwrite, works
-    job = KIO::linkAs(QUrl::fromLocalFile(sourceFile + "2"), QUrl::fromLocalFile(dest), KIO::Overwrite | KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    job = KIO::linkAs(QUrl::fromLocalFile(sourceFile + QLatin1Char('2')), QUrl::fromLocalFile(dest), KIO::Overwrite | KIO::HideProgressInfo);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFileInfo(dest).isSymLink());
-    QCOMPARE(QFileInfo(dest).symLinkTarget(), QString(sourceFile + "2"));
+    QCOMPARE(QFileInfo(dest).symLinkTarget(), QString(sourceFile + QLatin1Char('2')));
 
     // Changing the link target using KIO::symlink, with overwrite, works
-    KIO::Job *symlinkJob = KIO::symlink(sourceFile + "3", QUrl::fromLocalFile(dest), KIO::Overwrite | KIO::HideProgressInfo);
-    QVERIFY(symlinkJob->exec());
+    KIO::Job *symlinkJob = KIO::symlink(sourceFile + QLatin1Char('3'), QUrl::fromLocalFile(dest), KIO::Overwrite | KIO::HideProgressInfo);
+    QVERIFY2(symlinkJob->exec(), qPrintable(symlinkJob->errorString()));
     QVERIFY(QFileInfo(dest).isSymLink());
-    QCOMPARE(QFileInfo(dest).symLinkTarget(), QString(sourceFile + "3"));
+    QCOMPARE(QFileInfo(dest).symLinkTarget(), QString(sourceFile + QLatin1Char('3')));
 
     // Cleanup
     QVERIFY(QFile::remove(dest));
@@ -2008,7 +1963,7 @@ void JobTest::createBrokenSymlink()
     const QString dest = homeTmpDir() + "testlink";
     QFile::remove(dest); // just in case
     KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    QVERIFY(job->exec());
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
     QVERIFY(QFileInfo(dest).isSymLink());
 
     // Second time fails (already exists)
@@ -2040,8 +1995,7 @@ void JobTest::multiGet()
     //connect(job, &KIO::MultiGetJob::result, [=] (long id) { qDebug() << "ID I got" << id;});
     //connect(job, &KJob::result, [this](KJob* ) {qDebug() << "END";});
 
-    bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
     QCOMPARE(spyResult.count(), 1);
     QCOMPARE(spyResultId.count(), numFiles);
@@ -2079,8 +2033,7 @@ void JobTest::cancelCopyAndCleanDest()
 
     const QString baseDir = homeTmpDir();
     const QString srcTemplate = baseDir + QStringLiteral("testfile_XXXXXX");
-    const QString destFile = baseDir + QStringLiteral("testfile_copy");
-
+    const QString destFile = baseDir + QStringLiteral("testfile_copy_") + QString::fromLatin1(QTest::currentDataTag());
 
     QTemporaryFile f(srcTemplate);
     if (!f.open()) {
