@@ -72,6 +72,8 @@ private Q_SLOTS:
     void moveDirectoryToOtherPartition();
     void moveFileNoPermissions();
     void moveDirectoryNoPermissions();
+    void moveDirectoryToReadonlyFilesystem_data();
+    void moveDirectoryToReadonlyFilesystem();
     void deleteFile();
     void deleteDirectory();
     void deleteSymlink();
@@ -97,6 +99,8 @@ private Q_SLOTS:
     void moveDestAlreadyExistsAutoRename_data();
     void moveDestAlreadyExistsAutoRename();
 
+    void copyDirectoryAlreadyExistsSkip();
+
     void safeOverwrite();
     void safeOverwrite_data();
     void moveAndOverwrite();
