@@ -22,7 +22,7 @@
 #define KDIROPERATOR_H_
 
 #include <QWidget>
-#include <QtCore/QStack>
+#include <QStack>
 #include <QStyleOptionViewItem>
 
 #include <ktoggleaction.h>
@@ -490,6 +490,34 @@ public:
      */
     bool onlyDoubleClickSelectsFiles() const;
 
+    /**
+     * Toggles whether setUrl is called on newly created directories.
+     * @since 5.62
+     */
+    void setFollowNewDirectories(bool enable);
+
+    /**
+     * @returns true if setUrl is called on newly created directories, false 
+     * otherwise. Enabled by default.
+     * @since 5.62
+     * @see setFollowNewDirectories
+     */
+    bool followNewDirectories() const;
+
+    /**
+     * Toggles whether setUrl is called on selected directories when a tree view
+     * is used.
+     * @since 5.62
+     */
+    void setFollowSelectedDirectories(bool enable);
+
+    /**
+     * @returns whether setUrl is called on selected directories when a tree
+     * view is used. Enabled by default.
+     * @since 5.62
+     */
+    bool followSelectedDirectories() const;
+
     /**
      * Creates the given directory/url. If it is a relative path,
      * it will be completed with the current directory.
@@ -550,7 +578,7 @@ public:
     void setupMenu(int whichActions);
 
     /**
-     * Reimplemented - allow dropping of files if @p b is true
+     * Reimplemented - allow dropping of files if @p b is true, defaults to true since 5.59
      * @param b true if the widget should allow dropping of files
      */
     virtual void setAcceptDrops(bool b);
@@ -655,9 +683,9 @@ protected:
      */
     virtual void setDirLister(KDirLister *lister);
 
-    void resizeEvent(QResizeEvent *event) Q_DECL_OVERRIDE;
+    void resizeEvent(QResizeEvent *event) override;
 
-    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
+    void keyPressEvent(QKeyEvent *event) override; // TODO KF6 REMOVE
 
     /**
      * Sets up all the actions. Called from the constructor, you usually
@@ -704,9 +732,9 @@ protected:
      */
     virtual void activatedMenu(const KFileItem &item, const QPoint &pos);
 
-    void changeEvent(QEvent *event) Q_DECL_OVERRIDE;
+    void changeEvent(QEvent *event) override;
 
-    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
+    bool eventFilter(QObject *watched, QEvent *event) override;
 
 public Q_SLOTS:
     /**
@@ -762,7 +790,7 @@ public Q_SLOTS:
     QString makeCompletion(const QString &);
 
     /**
-     * Tries to complete the given string (only completes directores).
+     * Tries to complete the given string (only completes directories).
      */
     QString makeDirCompletion(const QString &);
 
@@ -879,8 +907,8 @@ Q_SIGNALS:
 
     /**
      * Emitted when a file is highlighted or generally the selection changes in
-     * multiselection mode. In the latter case, @p item is 0L. You can access
-     * the selected items with selectedItems().
+     * multiselection mode. In the latter case, @p item is a null KFileItem.
+     * You can access the selected items with selectedItems().
      */
     void fileHighlighted(const KFileItem &item);
     void dirActivated(const KFileItem &item);
@@ -912,6 +940,12 @@ Q_SIGNALS:
      */
     void currentIconSizeChanged(int size);
 
+    /**
+     * Triggered when the user hit Enter/Return
+     * @since 5.57
+     */
+    void keyEnterReturnPressed();
+
 private:
     class Private;
     Private *const d;
@@ -920,6 +954,7 @@ private:
     Q_PRIVATE_SLOT(d, void _k_slotSimpleView())
     Q_PRIVATE_SLOT(d, void _k_slotTreeView())
     Q_PRIVATE_SLOT(d, void _k_slotDetailedTreeView())
+    Q_PRIVATE_SLOT(d, void _k_slotToggleAllowExpansion(bool))
     Q_PRIVATE_SLOT(d, void _k_slotToggleHidden(bool))
     Q_PRIVATE_SLOT(d, void _k_togglePreview(bool))
     Q_PRIVATE_SLOT(d, void _k_toggleInlinePreviews(bool))
@@ -930,6 +965,9 @@ private:
     Q_PRIVATE_SLOT(d, void _k_slotSortByType())
     Q_PRIVATE_SLOT(d, void _k_slotSortReversed(bool))
     Q_PRIVATE_SLOT(d, void _k_slotToggleDirsFirst())
+    Q_PRIVATE_SLOT(d, void _k_slotIconsView())
+    Q_PRIVATE_SLOT(d, void _k_slotCompactView())
+    Q_PRIVATE_SLOT(d, void _k_slotDetailsView())
     Q_PRIVATE_SLOT(d, void _k_slotToggleIgnoreCase())
     Q_PRIVATE_SLOT(d, void _k_slotStarted())
     Q_PRIVATE_SLOT(d, void _k_slotProgress(int))
